import { Unit } from 'aws-embedded-metrics';
import type { Context } from 'aws-lambda';
import { HyperLogLog } from 'streamcount';
import { DocumentationLanguage } from '../shared/language';
export declare function handler(event: InventoryCanaryEvent, context: Context): Promise<InventoryCanaryEvent>;
export declare function realHandler(event: InventoryCanaryEvent, host: IInventoryHost): Promise<InventoryCanaryEvent>;
export declare const enum Grain {
    PACKAGE_MAJOR_VERSIONS = "package major versions",
    PACKAGE_VERSION_SUBMODULES = "package version submodules",
    PACKAGE_VERSIONS = "package versions",
    PACKAGES = "packages"
}
export declare const enum DocumentationStatus {
    /**
     * There is no documentation for this package/version/submodule
     *
     * This status is theoretical, and signaled by the absence of any of the other statuses.
     */
    MISSING = "Missing",
    /**
     * This package does not support the given language.
     */
    UNSUPPORTED = "Unsupported",
    /**
     * This package has a corrupted JSII assembly so we can't generate
     * documentation for it.
     */
    CORRUPT_ASSEMBLY = "CorruptAssembly",
    /**
     * This package supports the given language and has documentation for it.
     */
    SUPPORTED = "Supported"
}
/**
 * We have a markdown or JSON file from a particular language
 *
 * Pattern:
 *
 * ```
 * data/<package>/v<version>/docs-[<submodule>-]<language>.<json|md>[.not-supported]
 * ```
 *
 * Try to identify it
 */
export declare function classifyDocumentationFile(key: string): FileClassification;
/**
 * Expected input event structure as passed by the state machine.
 */
export interface InventoryCanaryEvent {
    readonly continuationObjectKey?: string;
}
/**
 * Intermediate state stored between invocations of the inventory canary.
 *
 * The canary accumulates data into this structure as it runs, until it is
 * finished. This structure is serialized to JSON in its entirety, so it may
 * not exceed 512MB.
 *
 * We therefore have to be clever around what and how we store data.
 */
export interface InventoryCanaryState {
    readonly continuationToken: string | undefined;
    readonly packageCountEstimate: HyperLogLog;
    readonly packageVersionCountEstimate: HyperLogLog;
    readonly majorVersionCountEstimate: HyperLogLog;
    readonly submoduleCountEstimate: HyperLogLog;
    readonly indexedPackageStates: {
        readonly metadataPresentEstimate: HyperLogLog;
        readonly tarballPresentEstimate: HyperLogLog;
        readonly assemblyPresentEstimate: HyperLogLog;
        readonly unknownObjectsEstimate: HyperLogLog;
        readonly uninstallable: string[];
    };
    readonly perLanguage: Map<string, PerLanguageData>;
}
/**
 * This used to be a map of { Grain -> { Package -> DocumentationStatus }}, so that
 * at the end we could count all the packages of a particular status, and report on some.
 *
 * We no longer collect full lists; instead we only collect cardinalities for most documentation
 * statuses (except MISSING, which we will derive by subtracting the sum total of the other statuses
 * from the total number of packages we are expecting).
 */
export type PerLanguageData = Map<Grain, Map<DocumentationStatus, HyperLogLog>>;
type Serialized<T> = string & {
    _serialized: T;
};
export declare function serialize<T>(value: T): Serialized<T>;
export declare function deserialize<T>(value: Serialized<T>): T;
export declare function freshCanaryState(): InventoryCanaryState;
export declare function isHyperLogLog(x: any): x is HyperLogLog;
/**
 * All the functions we need to abstract over to unit test this
 */
export interface IInventoryHost {
    log(...xs: any[]): void;
    loadProgress(continuationObjectKey: string): Promise<InventoryCanaryState>;
    saveProgress(state: InventoryCanaryState): Promise<InventoryCanaryEvent>;
    queueReport(reportKey: string, packageVersions: string[]): void;
    uploadsComplete(): Promise<void>;
    relevantObjectKeys(continuationToken?: string): AsyncGenerator<[string[], string | undefined], void, void>;
    /** Whether it's time to stop this invocation and have the State Machine start another Lambda */
    timeToCheckpoint(): boolean;
    metricScope(block: (m: IMetrics) => Promise<void>): Promise<void>;
}
export interface IMetrics {
    setDimensions(dimensionSet: Record<string, string>): IMetrics;
    putMetric(key: string, value: number, unit?: Unit | string): IMetrics;
}
export declare class AwsInventoryHost implements IInventoryHost {
    private readonly context;
    private readonly scratchworkBucket;
    private readonly packageDataBucket;
    private reports;
    constructor(context: Context);
    log(...xs: any[]): void;
    timeToCheckpoint(): boolean;
    /**
     * List all objects in the bucket, yielding batches of up to 1000 keys. Also
     * yields the next continuation token if there is one.
     */
    relevantObjectKeys(continuationToken?: string): AsyncGenerator<[string[], string | undefined], void, void>;
    saveProgress(state: InventoryCanaryState): Promise<InventoryCanaryEvent>;
    loadProgress(continuationObjectKey: string): Promise<InventoryCanaryState>;
    queueReport(reportKey: string, packageVersions: string[]): void;
    uploadsComplete(): Promise<void>;
    metricScope(block: (m: IMetrics) => Promise<void>): Promise<void>;
}
export type FileClassification = {
    type: 'known-json';
    lang: DocumentationLanguage;
    supported: boolean;
    submodule?: string;
} | {
    type: 'corrupt-assembly';
    lang: DocumentationLanguage;
} | {
    type: 'ignore';
} | {
    type: 'unknown';
};
export {};
